#include <unistd.h> 
#include <stdlib.h>
#include <fcntl.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>

#define LOCKFILE "/var/run/daemon.pid"
#define LOCKMODE ( S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH )


int lock_reg( int fd, int cmd, int type, off_t offset, int whence, off_t len ) {
   struct flock lock;
   lock.l_type = type;     /* F_RDLCK, F_WRLCK, F_UNLCK */
   lock.l_start = offset;  /* byte offset, relative to l_whence */
   lock.l_whence = whence; /* SEEK_SET, SEEK_CUR, SEEK_END */
   lock.l_len = len;       /* #bytes (0 means to EOF) */
   return( fcntl( fd, cmd, &lock ) );
}

#define write_lock(fd, offset, whence, len) \
   lock_reg( fd, F_SETLK, F_WRLCK, offset, whence, len )

int already_running( void ) {
   int fd;
   char buf[ 16 ];
   fd = open( LOCKFILE, O_RDWR | O_CREAT, LOCKMODE );
   if( fd < 0 ) {
      syslog( LOG_ERR, "не возможно открыть %s: %s",
              LOCKFILE, strerror( errno ) );
      exit( 1 );
   }
   if( write_lock( fd, 0, SEEK_SET, 0 ) < 0 ) {
      if( errno == EACCES || errno == EAGAIN ) {
         close( fd );
         return( 1 );
      }
      syslog( LOG_ERR, "невозможно установить блокировку на %s: %s",
              LOCKFILE, strerror( errno ) );
      exit( 1 );
   }
   ftruncate( fd, 0 );
   sprintf( buf, "%ld", (long)getpid() );
   write( fd, buf, strlen( buf ) + 1 );
   return( 0 );
}

